#include <cppunit/TestCaller.h>
#include <cppunit/TestSuite.h>
#include <cppunit/ui/text/TestRunner.h>
#include <cppunit/extensions/HelperMacros.h>
#include <cppunit/extensions/TestFactoryRegistry.h>

#include <vamos/geometry/Rectangle.h>

using namespace Vamos_Geometry;

class Enclose : public CppUnit::TestFixture
{
  CPPUNIT_TEST_SUITE (Enclose);
  CPPUNIT_TEST (left);
  CPPUNIT_TEST (right);
  CPPUNIT_TEST (top);
  CPPUNIT_TEST (bottom);
  CPPUNIT_TEST (around);
  CPPUNIT_TEST (aspect);
  CPPUNIT_TEST_SUITE_END ();

  Rectangle* mp_box;

public:
  void setUp ()
  {
    mp_box = new Rectangle (1.0, 2.0, 3.0, -1.0);
  }

  void tearDown ()
  {
    delete mp_box;
  }

  void left ()
  {
    mp_box->enclose (Rectangle (0.0, 1.0, 1.0, 0.0));
    CPPUNIT_ASSERT_EQUAL (mp_box->left, 0.0);
    CPPUNIT_ASSERT_EQUAL (mp_box->top, 2.0);
    CPPUNIT_ASSERT_EQUAL (mp_box->right, 3.0);
    CPPUNIT_ASSERT_EQUAL (mp_box->bottom, -1.0);
  }
  void right ()
  {
    mp_box->enclose (Rectangle (2.0, 1.0, 4.0, 0.0));
    CPPUNIT_ASSERT_EQUAL (mp_box->left, 1.0);
    CPPUNIT_ASSERT_EQUAL (mp_box->top, 2.0);
    CPPUNIT_ASSERT_EQUAL (mp_box->right, 4.0);
    CPPUNIT_ASSERT_EQUAL (mp_box->bottom, -1.0);
  }
  void top ()
  {
    mp_box->enclose (Rectangle (2.0, 3.0, 3.0, 1.0));
    CPPUNIT_ASSERT_EQUAL (mp_box->left, 1.0);
    CPPUNIT_ASSERT_EQUAL (mp_box->top, 3.0);
    CPPUNIT_ASSERT_EQUAL (mp_box->right, 3.0);
    CPPUNIT_ASSERT_EQUAL (mp_box->bottom, -1.0);    
  }
  void bottom ()
  {
    mp_box->enclose (Rectangle (2.0, 1.0, 3.0, -2.0));
    CPPUNIT_ASSERT_EQUAL (mp_box->left, 1.0);
    CPPUNIT_ASSERT_EQUAL (mp_box->top, 2.0);
    CPPUNIT_ASSERT_EQUAL (mp_box->right, 3.0);
    CPPUNIT_ASSERT_EQUAL (mp_box->bottom, -2.0);    
  }
  void around ()
  {
    mp_box->enclose (Rectangle (0.0, 3.0, 4.0, -2.0));
    CPPUNIT_ASSERT_EQUAL (mp_box->left, 0.0);
    CPPUNIT_ASSERT_EQUAL (mp_box->top, 3.0);
    CPPUNIT_ASSERT_EQUAL (mp_box->right, 4.0);
    CPPUNIT_ASSERT_EQUAL (mp_box->bottom, -2.0);    
  }
  void aspect ()
  {
    CPPUNIT_ASSERT_DOUBLES_EQUAL (mp_box->aspect (), 0.667, 0.001);
  }
};

CPPUNIT_TEST_SUITE_REGISTRATION (Enclose);
int main ()
{
  CppUnit::TextUi::TestRunner runner;
  runner.addTest (CppUnit::TestFactoryRegistry::getRegistry ().makeTest ());
  runner.run ();
  return 0;
}
