#include <cppunit/TestCaller.h>
#include <cppunit/TestSuite.h>
#include <cppunit/ui/text/TestRunner.h>
#include <cppunit/extensions/HelperMacros.h>
#include <cppunit/extensions/TestFactoryRegistry.h>

#include "../include/vamos/geometry/Spline.h"

using namespace Vamos_Geometry;

class No_Points : public CppUnit::TestFixture
{
  CPPUNIT_TEST_SUITE (No_Points);
  CPPUNIT_TEST (test_interpolate);
  CPPUNIT_TEST (test_load_point);
  CPPUNIT_TEST (test_load_points);
  CPPUNIT_TEST_SUITE_END ();

  Spline* mp_spline;

public:
  void setUp ()
  {
    mp_spline = new Spline ();
  }

  void tearDown ()
  {
    delete mp_spline;
  }

  void test_interpolate ()
  {
    CPPUNIT_ASSERT_EQUAL (mp_spline->interpolate (0.0), 0.0);
    CPPUNIT_ASSERT_EQUAL (mp_spline->interpolate (1.0), 0.0);
    CPPUNIT_ASSERT_EQUAL (mp_spline->interpolate (-1.0), 0.0);
  }

  void test_load_point ()
  {
    mp_spline->load (Two_Vector (1.0, 2.0));
    CPPUNIT_ASSERT_EQUAL (mp_spline->interpolate (0.0), 2.0);
    CPPUNIT_ASSERT_EQUAL (mp_spline->interpolate (1.0), 2.0);
    CPPUNIT_ASSERT_EQUAL (mp_spline->interpolate (-1.0), 2.0);
  }

  void test_load_points ()
  {
    std::vector <Two_Vector> points;
    points.push_back (Two_Vector (1.0, 2.0));
    points.push_back (Two_Vector (2.0, 3.0));
    mp_spline->load (points);
    CPPUNIT_ASSERT_EQUAL (mp_spline->interpolate (1.0), 2.0);
    CPPUNIT_ASSERT_EQUAL (mp_spline->interpolate (2.0), 3.0);
  }
};

CPPUNIT_TEST_SUITE_REGISTRATION (No_Points);
int main ()
{
  CppUnit::TextUi::TestRunner runner;
  runner.addTest (CppUnit::TestFactoryRegistry::getRegistry ().makeTest ());
  runner.run ();
  return 0;
}
