#include <vamos/geometry/Two_Vector.h>

#include <cppunit/TestCaller.h>
#include <cppunit/TestSuite.h>
#include <cppunit/ui/text/TestRunner.h>
#include <cppunit/extensions/HelperMacros.h>
#include <cppunit/extensions/TestFactoryRegistry.h>

#include <string>
#include <sstream>

using Vamos_Geometry::Two_Vector;

class Default : public CppUnit::TestFixture
{
  CPPUNIT_TEST_SUITE (Default);
  CPPUNIT_TEST (components);
  CPPUNIT_TEST_SUITE_END ();

  Two_Vector* null;

public:
  void setUp ()
  {
    null = new Two_Vector ();
  }

  void tearDown ()
  {
    delete null;
  }

  void components ()
  {
    CPPUNIT_ASSERT_EQUAL (null->x, 0.0);
    CPPUNIT_ASSERT_EQUAL (null->y, 0.0);
  }
};

class Math : public CppUnit::TestFixture
{
  CPPUNIT_TEST_SUITE (Math);
  CPPUNIT_TEST (add);
  CPPUNIT_TEST (subtract);
  CPPUNIT_TEST (multiply);
  CPPUNIT_TEST (divide);
  CPPUNIT_TEST_SUITE_END ();

  Two_Vector* v1;
  Two_Vector* v2;

public:
  void setUp ()
  {
    v1 = new Two_Vector (1.0, 2.0);
    v2 = new Two_Vector (3.0, 4.0);
  }

  void tearDown ()
  {
    delete v1;
    delete v2;
  }

  void add ()
  {
    Two_Vector v3 = *v1 + *v2;
    CPPUNIT_ASSERT_EQUAL (v3.x, 4.0);
    CPPUNIT_ASSERT_EQUAL (v3.y, 6.0);
  }

  void subtract ()
  {
    Two_Vector v3 = *v1 - *v2;
    CPPUNIT_ASSERT_EQUAL (v3.x, -2.0);
    CPPUNIT_ASSERT_EQUAL (v3.y, -2.0);
  }

  void multiply ()
  {
    Two_Vector v3 = *v1 * 3;
    CPPUNIT_ASSERT_EQUAL (v3.x, 3.0);
    CPPUNIT_ASSERT_EQUAL (v3.y, 6.0);

    v3 = 3 * *v2;
    CPPUNIT_ASSERT_EQUAL (v3.x, 9.0);
    CPPUNIT_ASSERT_EQUAL (v3.y, 12.0);
  }

  void divide ()
  {
    Two_Vector v3 = *v1 / 2.0;
    CPPUNIT_ASSERT_EQUAL (v3.x, 0.5);
    CPPUNIT_ASSERT_EQUAL (v3.y, 1.0);
  }
};

class Stream : public CppUnit::TestFixture
{
  CPPUNIT_TEST_SUITE (Stream);
  CPPUNIT_TEST (input);
  CPPUNIT_TEST (output);
  CPPUNIT_TEST_SUITE_END ();

public:
  void input ()
  {
    std::string s = "[ 1.2, -3.4 ]";
    std::istringstream is (s);
    Two_Vector v;
    is >> v;
    CPPUNIT_ASSERT_EQUAL (v.x, 1.2);
    CPPUNIT_ASSERT_EQUAL (v.y, -3.4);
  }

  void output ()
  {
    std::string s = "[ -2.2, 3.3 ]";
    std::ostringstream os;
    Two_Vector v (-2.2, 3.3);
    os << v;
    CPPUNIT_ASSERT_EQUAL (os.str (), s);
  }
};

CPPUNIT_TEST_SUITE_REGISTRATION (Default);
CPPUNIT_TEST_SUITE_REGISTRATION (Math);
CPPUNIT_TEST_SUITE_REGISTRATION (Stream);
int main ()
{
  CppUnit::TextUi::TestRunner runner;
  runner.addTest (CppUnit::TestFactoryRegistry::getRegistry ().makeTest ());
  runner.run ();
  return 0;
}
