#include <cppunit/TestCaller.h>
#include <cppunit/TestSuite.h>
#include <cppunit/ui/text/TestRunner.h>
#include <cppunit/extensions/HelperMacros.h>
#include <cppunit/extensions/TestFactoryRegistry.h>

#include "../include/vamos/geometry/XML_Parser.h"

#include <string>

using namespace Vamos_Geometry;

class Empty_Path : public CppUnit::TestFixture
{
  CPPUNIT_TEST_SUITE (Empty_Path);
  CPPUNIT_TEST (test_empty);
  CPPUNIT_TEST (test_push);
  CPPUNIT_TEST (test_push_two);
  CPPUNIT_TEST (test_push_drop);
  CPPUNIT_TEST (push_two_drop);
  CPPUNIT_TEST (test_top);
  CPPUNIT_TEST_SUITE_END ();

  XML_Path* mp_path;

public:
  void setUp ()
  {
    mp_path = new XML_Path ();
  }

  void tearDown ()
  {
    delete mp_path;
  }

  void test_empty ()
  {
    CPPUNIT_ASSERT (mp_path->empty ());
    CPPUNIT_ASSERT_EQUAL (mp_path->path (), std::string (""));
  }

  void test_push ()
  {
    mp_path->push ("element");
    CPPUNIT_ASSERT_EQUAL (mp_path->path (), std::string ("/element"));
  }

  void test_push_two ()
  {
    mp_path->push ("one");
    mp_path->push ("two");
    CPPUNIT_ASSERT_EQUAL (mp_path->path (), std::string ("/one/two"));
  }

  void test_push_drop ()
  {
    mp_path->push ("element");
    mp_path->drop ();
    CPPUNIT_ASSERT_EQUAL (mp_path->path (), std::string (""));
  }

  void push_two_drop ()
  {
    mp_path->push ("one");
    mp_path->push ("two");
    mp_path->drop ();
    CPPUNIT_ASSERT_EQUAL (mp_path->path (), std::string ("/one"));
  }

  void test_top ()
  {
    mp_path->push ("one");
    mp_path->push ("two");
    CPPUNIT_ASSERT_EQUAL (mp_path->top (), std::string ("two"));
  }
};

class Match_Path : public CppUnit::TestFixture
{
  CPPUNIT_TEST_SUITE (Match_Path);
  CPPUNIT_TEST (test_match);
  CPPUNIT_TEST_SUITE_END ();

  XML_Path* mp_path;

public:
  void setUp ()
  {
    mp_path = new XML_Path ();
  }

  void tearDown ()
  {
    delete mp_path;
  }

  void test_match ()
  {
    mp_path->push ("one");
    mp_path->push ("two");
    mp_path->push ("three");
    mp_path->push ("four");
    CPPUNIT_ASSERT (mp_path->match ("/one/two/three/four"));
    CPPUNIT_ASSERT (!mp_path->match ("/one/two/five"));
    CPPUNIT_ASSERT (!mp_path->match ("/one/two/"));
    CPPUNIT_ASSERT (!mp_path->match ("three/four"));
    CPPUNIT_ASSERT (mp_path->match ("/one/two/*"));
    CPPUNIT_ASSERT (!mp_path->match ("/one/three/*"));
    CPPUNIT_ASSERT (mp_path->match ("*/three/four"));
    CPPUNIT_ASSERT (!mp_path->match ("*/one/three"));
    CPPUNIT_ASSERT (mp_path->match ("/one/t*ree/four"));
    CPPUNIT_ASSERT (!mp_path->match ("/one/t*six"));
    CPPUNIT_ASSERT (!mp_path->match ("/one/two/*two/three/four"));
    CPPUNIT_ASSERT (mp_path->match ("/one/*/thr*our"));
    CPPUNIT_ASSERT (mp_path->match ("/one/two/th**ree/four"));
    CPPUNIT_ASSERT (mp_path->match ("/one/t*three/four"));
  }
};

CPPUNIT_TEST_SUITE_REGISTRATION (Empty_Path);
CPPUNIT_TEST_SUITE_REGISTRATION (Match_Path);
int main ()
{
  CppUnit::TextUi::TestRunner runner;
  runner.addTest (CppUnit::TestFactoryRegistry::getRegistry ().makeTest ());
  runner.run ();
  return 0;
}
